%macro di_etl_call_webservice(_method=,resume=0, status_log_file=, debug=, _user=, _pwd=, 
                              _authDomain=, _url=, _encoded=YES, _getPWFromDomain=YES, 
                              nightly=0, job_process_dt=0, scheduled_flg=, tkmi_args=, application=) ;


/***************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_etl_call_webservice

DESCRIPTION:   This macro generates code and calls a web service.
               URL where webservice resides is in a global settings table
               AuthDomain which has permissions to call web service is also in global settings table

APPLICATIONS:  All
MODULE:        ETL

INPUTS:        _method - Method which needs to be called.  The following are
               valid values.  This list could expand later.
                 etlStart
                 copyAndStartEstimation
                 waitForEstimation
                 etlEnd
                 copyDataToCaptains
                 
               resume This is used to resume copyAndStartEstimation and etlEnd process.
               status_log_file  Log file to which status messages should be sent.  Not used in all the methods.
               debug  A value of 1 prints some additional debug messages
               _user  User name
               _pwd   password
               _authDomain Authentication domain
               _url    Web service URL
               _encoded Password is encoded or not.  It is encoded in the code if it is not
               _getPWFromDomain  If user id and password should be retrieved from metadata server  

OUTPUTS:       Error Code and Description of Error if any

GLOBAL VAR:    NA
USAGE:         
THREADED:      NA
DOCUMENT:      NA
COMMENTS:
AUTHOR:        Venu Kadari
SUPPORT:       vekada Kadari, Venu
HISTORY:       28Jan2010 vekada   Initial Write
               12Apr2010 vekada   ANALYTICAL_PROD_PARTITION_LVL and PLANNING_PROD_PARTITION_LVL changed to PROD_PARTITION_LVL
               23Aug2010 vekada   S0664347  Add a parameter called "nightly" to this macro.  This parameter has a value
                                  0 by default.  When it is non-zero, it impacts etlEnd soapCall.  Include the following line
                                  <overrideArg xsi:type="xsd:string">batch_opt_nightly</overrideArg>
               14Sep2010 sinprs   Add scheduled_flg argument for copyAndStartEstimation. No default value. This could be a 
                                  comma separated list of numbers.        
                                  resume will be used for batch_est, batch_opt, batch_opt_nightly, batch_prep.
                                  These two arguments are combined in cmdOptions, which is a new argument in Data Services.
               04May2011 sinprs   Use Product_hier_assoc_dm to fetch valid partitions because some products may be in Product_dm
                                  but may not exist in the hierarchy (such as dangling nodes).                 
               28Jun2011 sinprs   Add job_process_dt to nightly optimization; The will be passed from Grid Server to 
                                  batch_prep command with nightly=1 .
               29Jul2011 sinprs   Add debug option to CmdOptions if requested from tkmi_job_ro()         
                                  Debug option should be limited to etlEnd and CopyAndStartEstimation
               12Feb2013 sinprs   Add tkmi_args to allow specifying run time arguments while calling tkmi.
	       15Feb2017 sinsro   S1313319 Add application parameter to run the TKMIServer code only for specified app

****************************************************************************************/

   %local m_aKs; /* Analytical Keys */
   %local m_AH_Code;
   %local m_AH_Level;
   %local m_AuthDomain;

   %let nightly=&nightly;
   %let scheduled_flg = %bquote(&scheduled_flg);
   
   %if &resume eq %str() or &resume eq %str(.) %then %let resume=0;
   %if &nightly eq %str() or &nightly eq %str(.) %then %let nightly=0;
   %if &job_process_dt eq %str() or &job_process_dt eq %str(.) %then %let job_process_dt=0;

   %let m_authDomain=&_authDomain; 
   
   * If authDomain is blank then get it from SYSTEM variable ;
   %if "&m_authDomain"="" %then 
   %do;
      %let m_authDomain=%SYSGET(ETL_WEBSERVICE_AUTHDOMAIN);
      %let m_authDomain=&m_authDomain; 
   %end;   
   
   * Get authDomain from global settings if still blank;
   %if "&m_authDomain"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_AuthDomain
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_AUTHDOMAIN' 
            and upcase(app) = 'ALL';

      quit;
      %let m_authDomain=&m_authDomain; 
   %end;   
   
   
   %let m_user=&_user; 
   
   * Get user from global settings if still blank;
   %if "&m_user"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_user
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_USER' 
            and upcase(app) = 'ALL';

      quit;
      %let m_user=&m_user; 
   %end;
      
   * If user is blank then get it from SYSTEM variable ;
   %if "&m_user"="" %then 
   %do;
      %let m_user=%SYSGET(ETL_WEBSERVICE_USER);
      %let m_user=&m_user; 
   %end;   
   
   %if "&m_user"="" %then 
   %do;
      %let m_user=&SYSUSERID;
      %let m_user=&m_user; 
   %end;   
   
   %let m_pwd=&_pwd; 
   
   * Get pwd from global settings if still blank;
   %if "&m_pwd"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_pwd
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_PWD' 
            and upcase(app) = 'ALL';

      quit;
      %let m_pwd=&m_pwd; 
   %end;   
   
   * If pwd is blank then get it from SYSTEM variable ;
   %if "&m_pwd"="" %then 
   %do;
      %let m_pwd=%SYSGET(ETL_WEBSERVICE_PWD);
      %let m_pwd=&m_pwd; 
   %end;   
   
   %let m_url=&_url; 
   
   * If url is blank then get it from SYSTEM variable ;
   %if "&m_url"="" %then 
   %do;
      %let m_url=%SYSGET(ETL_WEBSERVICE_url);
      %let m_url=&m_url; 
   %end;   
   
   * Get url from global settings if still blank;
   %if "&m_url"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_url
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_URL' 
            and upcase(app) = 'ALL';

      quit;
      %let m_url=&m_url; 
   %end;   
   
   

/**
   Get PROD_PARTITION_LVL 
*/

   proc sql noprint;

        select spec_value 
           into :m_AH_Level
           from di_data.global_setting
        where upcase(spec_nm) = 'PROD_PARTITION_LVL' 
        and upcase(app) = 'ALL';
   quit;

  
   
   %put NOTE: Web service URL=&m_url;
   %put NOTE: User ID=&m_user;
   %put NOTE: Authentication Domain=&m_AuthDomain;
   %put NOTE: Planning hierarchy level=&m_AH_Level;
   %put NOTE: Application=&application;

   %if ((NOT %symexist(m_authDomain)) or (%length(&m_authDomain)<2)) %then 
   %do;
      %put NOTE: Web service authDomain macro variable not specified. ;
      %put NOTE-: Specify _authDomain=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_AUTHDOMAIN as system variable or global setting.;
      %let m_authDomain=defaultAuth;
   %end;

   %if ((NOT %symexist(m_user)) or (%length(&m_user)<2)) %then 
   %do;
      %put NOTE: Web service user macro variable not specified. ;
      %put NOTE-: Specify _user=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_USER as system variable or global setting.;
   %end;

   %if ((NOT %symexist(m_pwd)) or (%length(&m_pwd)<2)) %then 
   %do;
      %put NOTE: Web service pwd macro variable not specified. ;
      %put NOTE-: Specify _pwd=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_pwd as system variable or global setting.;
   %end;

   %if ((NOT %symexist(m_url)) or (%length(&m_url)<2)) %then 
   %do;
      %put NOTE: Web service url not specified. ;
      %put NOTE-: Specify _url=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_URL as system variable or global setting.;
   %end;

    %if %upcase(&_method) = ETLSTART %then %do;
       %let _method=etlStart;
    %end;
    %else %if %upcase(&_method) = COPYANDSTARTESTIMATION %then %do;
       %let _method=copyAndStartEstimation;
    %end;
    %else %if %upcase(&_method) = COPYDATATOCAPTAINS %then %do;
       %let _method=copyDataToCaptains;
    %end;
    %else %if %upcase(&_method) = WAITFORESTIMATION %then %do;
       %let _method=waitForEstimation;
    %end;
    %else %if %upcase(&_method) = ETLEND  %then %do;
       %let _method=etlEnd;
    %end;
    %else %if %upcase(&_method) = LOADCAPTAINS  %then %do;
       %let _method=loadCaptains;
    %end;
    %else     
    %do; 
         %put ERROR: Method "&_method" is unknown.  etlStart, copyAndStartEstimation, waitForEstimation, copyDataToCaptains and  etlEnd are known methods.;
    %end;


   /* Determine Analytical Hierarchy Code */
   %if ((%upcase(&_method) =COPYANDSTARTESTIMATION ) or
           (%upcase(&_method) =COPYDATATOCAPTAINS )) %then 
   %do;

      %if &m_AH_Level= %then 
      %do; 
         %put ERROR: The column SPEC_VALUE for SPEC_NM=PROD_PARTITION_LVL in Global_Setting table is blank or this row is missing.;
      %end;
      /* 
         UsePlanningHierarcy is always YES. UsePlanningHierarchy=NO is no longer valid.
         This was used for Analytical Hierarchy subsetting 
      */
      %getAnalyticalKeys(UsePlanningHierarchy=YES);
 
  %end;


   /* Determine Planning Hierarchy Code */
   %if %upcase(&_method) = ETLEND %then 
   %do;
      %getPlanningKeys();
 
  %end;

 
   %let m_encoded=&_encoded;
   %let AuthDomain=&m_AuthDomain;
   %let UserID=&m_user;
   %let PASSWORD=&m_pwd;

   %if &_getPWFromDomain=YES and "&m_pwd"="" %then
   %do;
      %di_getpwfromdomain(&AuthDomain,NO,USERID,PASSWORD);
      %put NOTE: Done with di_getpwfromdomain;
      %let m_user=&UserID;
      %let m_pwd=&password;
   %end;   


   /* TODO: we need a more robust check for encoded, check if the password begins with {sas */
   %if "&m_encoded"="YES" %then 
   %do;
      %let dbpass=&m_pwd;
   %end;
   
   %else
   %do;
      filename pwfile TEMP; 

      proc pwencode in="&m_pwd" out=pwfile;
      run;


     /* save password to this macro variable */
     %local dbpass;

     /* Get macro variable from file */
      data _null_;
         infile pwfile obs=1 length=l; 
         input @;
         input @1 line $varying1024. l; 
         call symput('dbpass',substr(line,1,l)); 
      run;
      
       filename pwfile;
   %end;
    
   %put dbpass=&dbpass;

   /*Save to root folder if debug on.  Otherwise in temp folder */

   %if &debug=1 %then %do;

      FILENAME ACTION    './action.xml';
      FILENAME request   './request.xml'; 
      FILENAME RESPONSE  './response.xml'; 
   %end;
   %else %do;

      FILENAME ACTION   TEMP ;
      FILENAME request  TEMP ; 
      FILENAME RESPONSE  TEMP; 

   %end;


   %local m_AH cmdOptions;
   /* m_log is set to N if no log file is in the parameters of the method being called.  It is N for COPYDATATOCAPTAINS */
   %let m_log=Y;

   /* action.xml contains only one line with SOAPAction:Ignore */
   DATA _NULL_;
     FILE ACTION;
     PUT ACTION @1 "SOAPAction:Ignore";
   run;

   /* Prepare REQUEST File */
   DATA _NULL_;
      FILE request LRECL=32000;
      length temp $32000;
      PUT request @1 '<?xml version="1.0" encoding="UTF-8"?>';                                          
      PUT request @1 '<soapenv:Envelope';                                                               
      PUT request @1 ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"';                         
      PUT request @1 ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"';                                   
      PUT request @1 ' xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"';                      
      PUT request @1 ' xmlns:data="http://data.tkmiserver.di.solutions.sas.com">';                                   
      PUT request @1 ' <soapenv:Header/>';                                                                 
      PUT request @1 ' <soapenv:Body>';                                                                 
      PUT request @1 "  <data:&_method soapenv:encodingStyle=""http://schemas.xmlsoap.org/soap/encoding/"">";    

      %if (%upcase(&_method) =COPYDATATOCAPTAINS ) %then 
      %do;
         %let m_log=N;
      %end;
            
      %if ((%upcase(&_method) =COPYANDSTARTESTIMATION ) or
           (%upcase(&_method) =COPYDATATOCAPTAINS ) or
           (%upcase(&_method) =ETLEND )) %then %do;

         PUT request @1 &m_AH_Code;
            temp=symget('m_AKs');
            put request @1 temp;
         
      %end;
      
      /* Only for nightly and etlEnd */
      %if ((%upcase(&_method) =ETLEND) and ("&nightly" ne "0" ) ) %then 
      %do;
          *PUT request @1 "   <overrideArg xsi:type=""xsd:string"">batch_opt_nightly</overrideArg>"; 
          %let cmdOptions = %str(nightly=%bquote(&nightly);job_process_dt=%bquote(&job_process_dt));
	  %end;
      
      /* Only if we need to send log file */
      %if &m_log=Y %then %do;
          PUT request @1 "   <fullLogPath xsi:type=""xsd:string"">&status_log_file</fullLogPath>"; 
      %end;

      /* Use scheduled_flg only for CopyAndStartEstimation method */
      %if ((%upcase(&_method) =COPYANDSTARTESTIMATION) and ("&scheduled_flg" ne "")) %then 
      %do;
          %let cmdOptions = %str(scheduled_flg=%bquote(&scheduled_flg));      
      %end;
      
      %if "&tkmi_args" ne "" %then %do;
         %if %length(&cmdOptions) gt 0 %then %do;
            %let cmdOptions = %str(&cmdOptions;&tkmi_args);
         %end;
         %else %do;
            %let cmdOptions = %str(&tkmi_args);
         %end;
      %end;
      /* Add debug option to command line if either 0 or 1 */
      %if %eval(&debug eq 1) or %eval(&debug eq 0) %then %do;
         %if %length(&cmdOptions) gt 0 %then %do;
            %let cmdOptions = %str(&cmdOptions;debug=&debug);
         %end;
         %else %do;
            %let cmdOptions = %str(debug=&debug);
         %end;         
      %end;

      %let cmdOptions = %str(application=%bquote(&application);&cmdOptions);
	  
      /* Use resume for CopyAndStartEstimation and etlEnd methods */
      %if (%upcase(&_method)=COPYANDSTARTESTIMATION or %upcase(&_method) =ETLEND ) %then 
      %do;
         PUT request @1 "   <resume xsi:type=""xsd:int"">&resume</resume>";
         PUT request @1 "   <cmdOptions xsi:type=""xsd:string"">&cmdOptions</cmdOptions>";            
      %end;
      
      PUT request @1 "  </data:&_method>";                                                                                                                                               
      PUT request @1 ' </soapenv:Body>';                                                                                                                                                
      PUT request @1 '</soapenv:Envelope>';                                                                                                                                             
   RUN;

   /* Delete Response file before submitting this call.  Just so we do not capture old response */
   %if %sysfunc(fileexist(response)) %then
   %do;
      %put deleting response file...;
      %let sysrc=%sysfunc(fdelete(response)); 
   %end;

   /* Make Proc HTTP Call */
   proc http 
      in=request
      out=response 
      method="post"
      headerin=ACTION 
      url="&m_url"
      webusername="&m_user"
      webpassword="&dbpass"
      webdomain="&m_AuthDomain"
      ;
   run;

   /* Write Request and Response to the Log */
   data _null_;
      infile REQUEST ;
      input;
      put _infile_;
   run;

   %if %sysfunc(fileref(RESPONSE)) eq 0 %then %do;
      data _null_;
         infile RESPONSE ;
         input;
         put _infile_;
      run;
   %end;   


   /* update tot_units_processed */
   %if (%upcase(&_method) = WAITFORESTIMATION) %then %do;
     %grep_units_processed(run=batch_est, _units=tot_units_processed);
   %end;
   %else %if (%upcase(&_method) = ETLEND) %then %do;
     %local val;
     %grep_units_processed(run=batch_opt, _units=tot_units_processed);
     %grep_units_processed(run=batch_prep, _units=val);
     %if (&val > &tot_units_processed) %then %let tot_units_processed = &val;
   %end;
   

%mend;


/*  Macro to store XML for Analytical Keys into a macro variable */
 %macro getAnalyticalKeys(UsePlanningHierarchy=);


     /* Get Analytical Keys 
        Select prod_hier_sk where Prod_LVL is same as Analytical Hierarchy Level.
     */

     %if &UsePlanningHierarchy=YES %then 
     %do;

         %let m_AH_Code='     <analyticHierarchyCode xsi:type="xsd:int">1</analyticHierarchyCode>';

         proc sql;
            create table work.AH_Keys as
            select distinct Prod_hier_sk 
              from di_data.PRODUCT_HIER_ASSOC_DM
             where PROD_HIER_ASSOC_CD = 1
               and PROD_LVL=&m_AH_Level
             ;
         quit;
     %end;
     %else %do;

         %let m_AH_Code='     <analyticHierarchyCode xsi:type="xsd:int">0</analyticHierarchyCode>';

         proc sql;
            create table work.AH_Keys as
            select Prod_hier_sk 
              from di_data.ATTR_PROD_DM
             where PROD_LVL=&m_AH_Level
             ;
         quit;
     %end;


   data _null_;

      set work.AH_Keys NOBS=NOBS END=END;

      attrib key_val length=$80;
      key_val=put(prod_hier_sk, 8.);
*      key_val=transtrn(key_val,"-","n");

      /* Analytical Keys Header */
      if _n_=1 then do;
         call symput('m_AKs', BYTE(10) || BYTE(13)    );
         call symput('m_AKs', symget('m_AKs') || '         <analyticalKeys soapenc:arrayType="xsd:long[' || compress(put(NOBS,4.)) || ']"' || BYTE(10) || BYTE(13)    );
         call symput('m_AKs', symget('m_AKs') || '               xsi:type="soapenc:Array" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || BYTE(10) || BYTE(13)    ); 
      end;

      row=compress(put(_n_-1,4.));
      call symput('m_AKs', symget('m_AKs') || '            <analyticalKeys' || compress(row) || ' xsi:type="xsd:long">' || compress(put(key_val,8.)) || '</analyticalKeys' || compress(row) || '>' || BYTE(10) || BYTE(13)   );

      /* End of Analytical Keys */
      if END then do;
         call symput('m_AKs', symget('m_AKs') || '         </analyticalKeys>' || BYTE(10) || BYTE(13)   );
         call symput('m_AKs', symget('m_AKs') || '         <analyticalLevel xsi:type="xsd:int">' || compress(&m_AH_Level) || '</analyticalLevel>' || BYTE(10) || BYTE(13) );
      end;


   run;


%mend getAnalyticalKeys;




/*  Macro to store XML for Planning Keys into a macro variable */
 %macro getPlanningKeys();

      /* Planning Hierarchy Code is always 1 */
      %let m_AH_Code='     <planningHierarchyCode xsi:type="xsd:int">1</planningHierarchyCode>';

      /* Get Planning Keys 
         Select prod_hier_sk where Prod_LVL is same as Planning Hierarchy Level.
      */
      
      proc sql;
         create table work.AH_Keys as
         select distinct Prod_hier_sk 
           from di_data.PRODUCT_HIER_ASSOC_DM
          where PROD_HIER_ASSOC_CD = 1
            and PROD_LVL=&m_AH_Level
          ;
      quit;
      
   %put NOTE: m_AH_Level=&m_AH_Level;

   
   data _null_;

      set work.AH_Keys NOBS=NOBS END=END;

      attrib key_val length=$80;
      key_val=put(prod_hier_sk, 8.);

      /* Planning Keys Header */
      if _n_=1 then do;
         call symput('m_AKs', BYTE(10) || BYTE(13)    );
         call symput('m_AKs', symget('m_AKs') || '         <planningKeys soapenc:arrayType="xsd:long[' || compress(put(NOBS,4.)) || ']"' || BYTE(10) || BYTE(13)    );
         call symput('m_AKs', symget('m_AKs') || '               xsi:type="soapenc:Array" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || BYTE(10) || BYTE(13)    ); 
      end;

      row=compress(put(_n_-1,4.));
      call symput('m_AKs', symget('m_AKs') || '            <planningKeys' || compress(row) || ' xsi:type="xsd:long">' || compress(put(key_val,8.)) || '</planningKeys' || compress(row) || '>' || BYTE(10) || BYTE(13)   );

      /* End of planning Keys */
      if END then do;
         call symput('m_AKs', symget('m_AKs') || '         </planningKeys>' || BYTE(10) || BYTE(13)   );
         call symput('m_AKs', symget('m_AKs') || '         <planningLevel xsi:type="xsd:int">' || compress(&m_AH_Level) || '</planningLevel>' || BYTE(10) || BYTE(13) );
      end;

   run;


%mend getPlanningKeys;



/* fetch Total Unit Processed from the current status log file */
%macro grep_units_processed(run=, _units=);

%if (not %sysfunc(fileexist(&di_job_status_log))) %then %do;
  %put &di_job_status_log does not exist;
  %let &_units = 0;
  %return;
%end;

%local count;
%let count = 0;

/*
      >Waiting for estimation
      >revopt09: batch_est (partition_id=0)
      >  Model_Group_Ids: 1
      >    20
      >  Success with DQ: 1
      >    20
      >  Min/Avg/Max Units Processed: 421/421/421
      >  Min/Avg/Max Elapse Time(s) : 1/1/1
      >  Total Units Processed: 421
      >  Total Elapse Time(s) : 2
*/

%if ((%index(&sysscp,WIN) ~= 0 or %index(&sysscp,DNT) ~= 0)) %then %do;
  option noxwait;
  %sysexec copy /y "%sysfunc(translate(&di_job_status_log,\,/))" "%sysfunc(pathname(work))\___units.log~";
%end;
%else %do;
  %sysexec cp "&di_job_status_log" "%sysfunc(pathname(work))/___units.log~";
%end;

filename fin "%sysfunc(pathname(work))/___units.log~";
%if (%sysfunc(fexist(fin))) %then %do;
   data _null_;
      length line $1024;
      infile fin lrecl=1024;
      input;
      line = _infile_;

      retain wait_for_units 10.0;

      if (_N_ = 1) then do;
         wait_for_units = 0;
      end;

      if (index(line, ">") > 0) then do;
         if (index(line, "&run") > 0) then do;
            wait_for_units = 1;
         end;
         else if (wait_for_units) then do;
           pos = index(line, "Total Units Processed:");
           if (pos > 0) then do;
              pos = pos + length("Total Units Processed:");
              units = input(substr(line,pos), best.);
              if (not missing(units)) then do;
                count = input(symget('count'),best.);
                count = count + units;
                call symput('count', put(count, best.));
              end;

              wait_for_units = 0;
           end;
         end;
      end;
   run;
  
   %let count = &count;
%end;
filename fin clear;


%if ((%index(&sysscp,WIN) ~= 0 or %index(&sysscp,DNT) ~= 0)) %then %do;
  option noxwait;
  %sysexec del /f /q "%sysfunc(pathname(work))\___units.log~";
%end;
%else %do;
  %sysexec rm "%sysfunc(pathname(work))/___units.log~";
%end;

%let &_units = &count;

%mend;




/*   
options mlogic mprint symbolgen;
   
libname di_data '\\midata\data\official\5.2\tests_mini';
libname di_dm '\\midata\data\official\5.2\tests_mini';

options metaport =8561
metaserver = "des18.na.sas.com"
metauser = "miuser"
metauser = "sasdemo"
metapass = "DIAdmin1"
;

%let logFile=C:\dev\vert-a2mi52\Products\DemandIntelligence\Retail2\Server\TKMIServer\MockETLLog.log;
%di_etl_call_webservice(_method=etlStart,resume=, status_log_file=&logFile, debug=) ;
%di_etl_call_webservice(_method=copyAndStartEstimation,resume=, status_log_file=&logFile, debug=) ;
%di_etl_call_webservice(_method=copyDataToCaptains,resume=, status_log_file=&logFile, debug=) ;
%di_etl_call_webservice(_method=waitForEstimation,resume=, status_log_file=&logFile, debug=) ;
%di_etl_call_webservice(_method=etlEnd,resume=, status_log_file=&logFile, debug=,nightly=0) ;
%di_etl_call_webservice(_method=etlEnd,resume=, status_log_file=&logFile, debug=,nightly=1) ;



*/




 
