/****************************************************************************
                Merchandise Intelligence

NAME:           GetMIMetadata

DESCRIPTION:    Macro to retrieve metadata information required by the backend
                from metadata.

APPLICATIONS:   Merchandise Intelligence Solutions

INPUTS:         serverName - The name of the transactional database server
                             defined in metadata.

OUTPUTS:        Sets several macro variables containing the information found.

USAGE:          Called by autoexec.sas for MI to initialize backend information.

AUTHOR:         sasres Ron Stogner

SUPPORT: sasres Stogner, Ron

HISTORY:        100605 Creation (Adapted from "getCredentials" macro)
                21aug2018 defect: S1398329 

****************************************************************************/

%macro di_getmimetadata( serverName, encode, TLIB=DI_DM2 ) /*storesecure*/ ;

   %if (%length(%sysfunc(getoption(metaPort))) = 0 or
        %length(%sysfunc(getoption(metaServer))) = 0 or
        %length(%sysfunc(getoption(metaRepository))) = 0 or
        %length(&serverName) = 0) %then %do;
      %put ERROR: Cannot connect to metadata server, because not all meta options are defined.;
      %return;
   %end;   
   %let syscc = 0;

   /* Define macro variables used by the backend as global */

   %global DI_TRANS_USER DI_TRANS_PASSWORD DI_TRANS_DATABASE DI_TRANS_SCHEMA ;
   %global DI_TRANS_SERVER DI_TRANS_PORT DI_TRANS_VENDOR DI_DATA_PATH continue ;
   %global mi_common_lic mi_rpp_lic mi_rpo_lic mi_cpo_lic mi_szpf_lic mi_szpk_lic;

   /* Initialize them to empty */

   %let DI_TRANS_USER=;
   %let DI_TRANS_PASSWORD=;
   %let DI_TRANS_DATABASE=;
   %let DI_TRANS_SCHEMA=;
   %let DI_TRANS_SERVER=;
   %let DI_TRANS_PORT=;
   %let DI_TRANS_VENDOR=;
   %let continue=YES;

   /* Go fetch the information from metadata */

   data _null_;

      length svruri $200 srcconnuri $200 propuri $200 loginuri $200 dbname $200 userid $60 password $200;
      length schemauri $200 schemaname $200 id $ 17 vendor $ 20 server $ 200 uri $ 200 name $ 200 ;
      length i 8 n 8 j 8 n2 8 temp $ 200 liburi $ 200 ;

      userid=""; password="";
      svruri="omsobj:ServerComponent?@Name='&serverName'";

      /* Fetch the database vendor and store */

      rc=metadata_getattr(svruri, "ClassIdentifier", vendor);
      vendor = upcase(vendor) ;

      if   vendor = 'DB2UNXPC'  then vendor = 'DB2' ;
      else if vendor = 'ORACLE' then vendor = 'ORACLE' ;
      else vendor = '' ;

      call symput("DI_TRANS_VENDOR", trim(vendor) ) ;

      /* If this is DB2 or Oracle store the associated machine as the server */

      if vendor = 'DB2' or vendor = 'ORACLE' then do ;
         rc=metadata_getnass(svruri, "AssociatedMachine", 1, uri);
         if rc ne 1 then do;
            put "ERROR: Unable to locate associated machine for &serverName.. Please confirm server name.";
            call symput("continue","NO");
            stop;
         end;
         rc=metadata_getattr(uri, "Name", server);
         call symput("DI_TRANS_SERVER", trim(server) ) ;
      end ;

      /* Get the associated "SourceConnections" for this server */

      rc=metadata_getnass(svruri, "SourceConnections", 1, srcconnuri);
      if rc lt 1 then do;
         put "ERROR: Unable to locate &serverName.. Please confirm server name.";
         call symput("continue","NO");
         stop;
      end;

      /* Get the "DataPackages" (has the DB schema, only for DB2) */

      if vendor = 'DB2' or vendor = 'ORACLE' then do;

         rc=metadata_getnass(svruri, "DataPackages", 1, schemauri);
         if rc lt 1 then do;
            put "ERROR: Unable to locate schema Please confirm server name.";
            call symput("continue","NO");
            stop;
         end;

         else if rc gt 1 then do ; /* Found multiple schemas, try and locate ours */
            n = rc ;
            do i = 1 to n ;
               rc=metadata_getnass(svruri, "DataPackages", i, schemauri);
               rc=metadata_getnass(schemauri, "UsedByPackages", 1, liburi);
               if rc ge 1 then do ;
                  n2 = rc ;
                  do j = 1 to n2 ;
                     temp="";
                     rc=metadata_getnass(schemauri, "UsedByPackages", j, liburi);
                     rc=metadata_getattr(liburi, "Libref", temp);
                     if trim(temp) = "&TLIB" then goto FoundSchema ;
                  end ; /* End of do "j" */
               end ; /* End of if */
            end ; /* End of do "i" */

            /* If we reach here, we did not find the matching schema! */
            put "ERROR: Unable to locate specific schema Please confirm server name.";
            call symput("continue","NO");
            stop;

         end ; /* End of if multiple schemas */

      end; /* End of if Vendor = 'DB2' or vendor = 'ORACLE' */

      FoundSchema:

      /* Get the authentication domain (has the userid and password) */

      rc=metadata_getnass(srcconnuri, "Domain", 1, loginuri);
      if rc ne 1 then do;
         put "ERROR: Unable to locate Authentication Domain for &serverName.. This is required to locate credentials.";
         call symput("continue","NO");
         stop;
      end;

     /* If the password is to be returned encoded, just fetch, otherwise more work to do. */

      %if &continue=YES and %upcase(&encode)=YES %then %do;
         rc=metadata_getnass(loginuri, "Logins", 1, loginuri);
         if rc=1 then do;
            rc=metadata_getattr(loginuri, "UserID", userid);
            call symput("DI_TRANS_USER", trim(userid)) ;
            rc=metadata_getattr(loginuri, "Password", password);
            call symput("DI_TRANS_PASSWORD", trim(password)) ;
         end;
      %end;
      %else %do;
         rc=metadata_getattr(loginuri, "Id", id);
         call symput("domainID", trim(id));
      %end;

      /* Get the "Properties" off the source connection (has the database name for DB2 */
      /* and the port, etc well for MySQL)                                               */

      rc=metadata_getnass(srcconnuri, "Properties", 1, propuri);
      if rc lt 1 then do;
         put "ERROR: Unable to locate Properties for &serverName.. Please confirm server name.";
         call symput("continue","NO");
         stop;
      end;
      n = rc ;

     /* Fetch the database name and schema for DB2, path and schema for Oracle */
     /* and the Server, Port and database for MySQL                            */

      if vendor='DB2' then do;
         rc=metadata_getattr(propuri, "DefaultValue", dbname);
         call symput( "DI_TRANS_DATABASE", trim(dbname) ) ;
         rc=metadata_getattr(schemauri, "SchemaName", schemaname);
         call symput( "DI_TRANS_SCHEMA", trim(schemaname) ) ;
      end;
      else if vendor = 'ORACLE' then do ;
         rc=metadata_getattr(propuri, "DefaultValue", dbname);
         call symput( "DI_TRANS_DATABASE", trim(dbname) ) ;
         rc=metadata_getattr(schemauri, "SchemaName", schemaname);
         call symput( "DI_TRANS_SCHEMA", trim(schemaname) ) ;
      end ;

      stop;

   run ;

   /* If we want clear text password, do that now */

   %if &continue=YES and %upcase(&encode) ne YES %then %do;
   
      /*
       * Retrieve the credentials from the Metadata
       */
      filename request temp;
      filename response temp lrecl=1024;
      filename map temp;
      /*
       *   Build the metadata request
       */
      data _null_;
         file request;
         put "<GetMetadata>";
         put "<Metadata>";
         put "%bquote(<AuthenticationDomain Id="&domainID">)";
         put "<Logins/>";
         put "</AuthenticationDomain>";  
         put "</Metadata>";
         put "<ns>SAS</ns>";
         put "<Flags>4</Flags>";
         put "<Options>";
         put "<Templates>";
         put "%bquote(<Login Name="""" Userid="""" Password="""">)";
         put "<AssociatedIdentity/>";
         put "</Login>";
         put "</Templates>";
         put "</Options>";
         put "</GetMetadata>";
      run;

      /*
       *   Issue the metadata request
       */
      proc metadata
         in=request
         out=response;
      run;

      /*
       *   Create Map for xml parsing
       */
      data _null_;
        file map;
        put '<SXLEMAP xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" name="SXLEMAP" version="1.2" xsi:noNamespaceSchemaLocation="http://www.sas.com/xml/schema/sxle12.xsd">';
        put '  <TABLE name="Logins">';
        put '        <TABLE-PATH syntax="XPath">/GetMetadata/Metadata/AuthenticationDomain/Logins</TABLE-PATH>';
        put '        <COLUMN name="Id">';
        put '            <PATH syntax="XPath">/GetMetadata/Metadata/AuthenticationDomain/Logins/Login/@Id</PATH>';
        put '            <TYPE>character</TYPE>';
        put '            <DATATYPE>string</DATATYPE>';
        put '            <LENGTH>17</LENGTH>';
        put '        </COLUMN>';
        put '        <COLUMN name="UserID">';
        put '            <PATH syntax="XPath">/GetMetadata/Metadata/AuthenticationDomain/Logins/Login/@UserID</PATH>';
        put '            <TYPE>character</TYPE>';
        put '            <DATATYPE>string</DATATYPE>';
        put '            <LENGTH>60</LENGTH>';
        put '        </COLUMN>';
        put '        <COLUMN name="Password">';
        put '            <PATH syntax="XPath">/GetMetadata/Metadata/AuthenticationDomain/Logins/Login/@Password</PATH>';
        put '            <TYPE>character</TYPE>';
        put '            <DATATYPE>string</DATATYPE>';
        put '            <LENGTH>200</LENGTH>';
        put '        </COLUMN>';
        put '    </TABLE>';
        put '</SXLEMAP>';
      run;
      libname response xml xmlmap=map;

      /*
       * retrieve credentials into macro variables
       */

      data _null_;
         set response.logins;
         call symput("DI_TRANS_USER", trim(userid)) ;
         call symput("DI_TRANS_PASSWORD", trim(password)) ;
         stop;
      run;

   %end;

   %let DI_TRANS_PASSWORD=%str(&DI_TRANS_PASSWORD);

   /*
    * fetch DI_DATA_PATH from DI_DM library in metadata
    */ 
   data _null_;
      length etl_name etl_uri_lib etl_uri_path etl_lib_path 
             etl_engine etl_isdbmslibname 
             etl_libref        $256;

      do until (etl_lrc < 0);
         call missing(of _all_);
         n + 1;
         etl_lrc = metadata_getnobj("omsobj:saslibrary?@name ? ''",n, etl_uri_lib); 
         if etl_lrc > 0 then do;
            rc = metadata_getattr(etl_uri_lib, 'libref', etl_libref); 
            rc = metadata_getattr(etl_uri_lib, 'name', etl_name);
            rc = metadata_getattr(etl_uri_lib, 'engine', etl_engine);
            rc = metadata_getattr(etl_uri_lib, 'isdbmslibname', etl_isdbmslibname); 
            rc = metadata_getnasn(etl_uri_lib, 'usingpackages', 1,etl_uri_path);
            rc = metadata_getattr(etl_uri_path, 'directoryname',etl_lib_path); 
            if (upcase(etl_libref) = "DI_DM") then do;
               call symput("DI_DATA_PATH", trim(etl_lib_path));
            end;
         end;
      end;
      stop;
   run;

   %if (&syscc > 4) %then %do;
      %put ERROR: Error during connecting to metadata server.;
   %end;

/* Validating Licenses of MI Products*/

%di_util_delete_tmp_folder(relative_dir = validate_lic,_return_code=_rc);
%let work_path = %di_util_create_tmp_folder(relative_dir = validate_lic);
filename setinit "&work_path./di_metasetinit.xml"; 

proc metadata in=
"<GetMetadataObjects>
   <Reposid>$METAREPOSITORY</Reposid>
   <Type>TextStore</Type>
   <NS>SAS</NS>
   <Flags>2441</Flags>
   <Options>
      <XMLSelect search=""*[@Name ? 'setinit']""/>
   </Options>
 </GetMetadataObjects>"  out=setinit;
run;

data _null_;
        file map;
        put '<SXLEMAP xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" name="SXLEMAP" version="1.2" xsi:noNamespaceSchemaLocation="http://www.sas.com/xml/schema/sxle12.xsd">';
        put '  <TABLE name="di_sas_lic">';
        put '        <TABLE-PATH syntax="XPath">GetMetadataObjects/Objects/TextStore</TABLE-PATH>';
        put '        <COLUMN name="StoredText">';
        put '            <PATH syntax="XPath">GetMetadataObjects/Objects/TextStore/@StoredText</PATH>';
        put '            <TYPE>character</TYPE>';
        put '            <DATATYPE>string</DATATYPE>';
        put '            <LENGTH>32000</LENGTH>';
        put '        </COLUMN>';
        put '    </TABLE>';
        put '</SXLEMAP>';
 run;

 libname setinit xml xmlmap=map;
	  
data work.di_sas_lic;
   length StoredText $32000;
   set setinit.di_sas_lic;
   StoredText=STRIP(StoredText);
run;

%let mi_common_lic=0;
%let mi_rpp_lic=0;
%let mi_rpo_lic=0;
%let mi_cpo_lic=0;
%let mi_szpf_lic=0;
%let mi_szpk_lic=0;


data _null_;
set work.di_sas_lic;
   if findw(StoredText,"'PRODNUM422'") > 0 then call symput('mi_common_lic',1);
   if findw(StoredText,"'PRODNUM428'") > 0 then call symput('mi_rpp_lic',1); 
   if findw(StoredText,"'PRODNUM426'") > 0 then call symput('mi_cpo_lic',1);
   if findw(StoredText,"'PRODNUM425'") > 0 then call symput('mi_rpo_lic',1);
   if findw(StoredText,"'PRODNUM429'") > 0 then call symput('mi_szpf_lic',1);
   if findw(StoredText,"'PRODNUM430'") > 0 then call symput('mi_szpk_lic',1);
run;

proc sql noprint;
drop table work.di_sas_lic;
quit;


%let mi_common_lic=&mi_common_lic;
%let mi_rpp_lic=&mi_rpp_lic;
%let mi_rpo_lic=&mi_rpo_lic;
%let mi_cpo_lic=&mi_cpo_lic;
%let mi_szpf_lic=&mi_szpf_lic;
%let mi_szpk_lic=&mi_szpk_lic;

%mend ;
