%macro di_tkmi_request(_cmd=, 
   prod_hier_sk=,
   prod_hier_sk_table=, 
   captainDir=,
   serverDir=,
   _rc=,
   debug=, 
   _user=, 
   _pwd=, 
   _authDomain=, 
   _url=, 
   _encoded=YES, 
   _getPWFromDomain=YES) ;

/***************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_tkmi_request

DESCRIPTION:   This macro generates code and calls a web service.
               URL where webservice resides is in a global settings table
               AuthDomain which has permissions to call web service is also in global settings table
               di_data.global_setting should have spec_value for spec_nm=ETL_WEBSERVICE_AUTHDOMAIN 
               di_data.global_setting should have spec_value for spec_nm=ETL_WEBSERVICE_URL 
                                 e.g. http://des18:1444/miserver/services/DataService
               

APPLICATIONS:  All
MODULE:        ETL

INPUTS:        _cmd             Method which needs to be called. TKMI extension command
                                   This is a required parameter.  
                                   e.g., fcst_export
               prod_hier_sk        Product Code
                                   Optional. Single product.  
               prod_hier_sk_table  Name of the table consisting of products
                                   Optional, allows to pass in multiple prod_hier_sks
               _rc                 Optional, return code =0 success ^= 0 failure     
               captainOutputDir    the output directory on each captain
               serverOutputDir     the output directory on the TKMI server
               _user  User name
               _pwd   password
               _authDomain Authentication domain
               _url    Web service URL
               _encoded Password is encoded or not.  It is encoded in the code if it is not
               _getPWFromDomain  If user id and password should be retrieved from metadata server  

OUTPUTS:       Error Code and Description of Error if any

GLOBAL VAR:    NA
USAGE:         
THREADED:      NA
DOCUMENT:      NA
COMMENTS:
AUTHOR:        Venu Kadari
SUPPORT:       vekada Kadari, Venu
HISTORY:       02JUN2010 vekada   Initial Write
               29Jul2011 sinprs   Add debug option to _cmd if this macro is called in debug mode 

****************************************************************************************/

   %local m_ProdSKs; /* Analytical Keys */
   %local m_AH_Code;

   %local m_AuthDomain;
   %local m_url;
   %let CRLF='0D0A'x;
   
   %let _len=%length(&_cmd);
   %let _lastchar=%sysfunc(substr(&_cmd,%length(&_cmd)));
   %put _lastchar=&_lastchar;
   
   /* Last character in the command must be a semi-colon */
   %if not ("&_lastchar"=";") %then 
   %do;
      %put WARNING: Semi-colon missing at the end of command passed in.  Command passed in is="&_cmd";
      %put WARNING-: Add a semi colon to the command "&_cmd";
   %end;
   
   /* Add debug option to command line if either 0 or 1 */
   %if %eval(&debug eq 1) or %eval(&debug eq 0) %then %do;
      %if %length(&_cmd) gt 0 %then %do;
         %let _cmd = %str(&_cmd.debug=&debug;);
      %end;
   %end;
   
   %let m_authDomain=&_authDomain; 
   
   * If authDomain is blank then get it from SYSTEM variable ;
   %if "&m_authDomain"="" %then 
   %do;
      %let m_authDomain=%SYSGET(ETL_WEBSERVICE_AUTHDOMAIN);
      %let m_authDomain=&m_authDomain; 
   %end;   
   
   * Get authDomain from global settings if still blank;
   %if "&m_authDomain"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_AuthDomain
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_AUTHDOMAIN' 
            and upcase(app) = 'ALL';

      quit;
      %let m_authDomain=&m_authDomain; 
   %end;   
   
   
   %let m_user=&_user; 
   
   * If user is blank then get it from SYSTEM variable ;
   %if "&m_user"="" %then 
   %do;
      %let m_user=&SYSUSERID;
      %let m_user=&m_user; 
   %end;   
   
   %if "&m_user"="" %then 
   %do;
      %let m_user=%SYSGET(ETL_WEBSERVICE_USER);
      %let m_user=&m_user; 
   %end;   
   
   * Get user from global settings if still blank;
   %if "&m_user"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_user
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_USER' 
            and upcase(app) = 'ALL';

      quit;
      %let m_user=&m_user; 
   %end;   
   
   
   %let m_pwd=&_pwd; 
   
   * If pwd is blank then get it from SYSTEM variable ;
   %if "&m_pwd"="" %then 
   %do;
      %let m_pwd=%SYSGET(ETL_WEBSERVICE_PWD);
      %let m_pwd=&m_pwd; 
   %end;   
   
   * Get pwd from global settings if still blank;
   %if "&m_pwd"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_pwd
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_PWD' 
            and upcase(app) = 'ALL';

      quit;
      %let m_pwd=&m_pwd; 
   %end;   
   
   
   %let m_url=&_url; 
   
   * If url is blank then get it from SYSTEM variable ;
   %if "&m_url"="" %then 
   %do;
      %let m_url=%SYSGET(ETL_WEBSERVICE_url);
      %let m_url=&m_url; 
   %end;   
   
   * Get url from global settings if still blank;
   %if "&m_url"="" %then 
   %do;
      proc sql noprint;
         select spec_value 
           into :m_url
           from di_data.global_setting
         where upcase(spec_nm) = 'ETL_WEBSERVICE_URL' 
            and upcase(app) = 'ALL';

      quit;
      %let m_url=&m_url; 
   %end;   
   
   %put NOTE: Web service URL=&m_url;
   %put NOTE: User ID=&m_user;
   %put NOTE: Authentication Domain=&m_AuthDomain;

   %if ((NOT %symexist(m_authDomain)) or (%length(&m_authDomain)<2)) %then 
   %do;
      %put NOTE: Web service authDomain macro variable not specified. ;
      %put NOTE-: Specify _authDomain=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_AUTHDOMAIN as system variable or global setting.;
      %let m_authDomain=defaultAuth;
   %end;

   %if ((NOT %symexist(m_user)) or (%length(&m_user)<2)) %then 
   %do;
      %put NOTE: Web service user macro variable not specified. ;
      %put NOTE-: Specify _user=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_USER as system variable or global setting.;
      %let m_user=sasdemo;
   %end;

   %if ((NOT %symexist(m_pwd)) or (%length(&m_pwd)<2)) %then 
   %do;
      %put WARNING: Web service pwd macro variable not specified. ;
      %put WARNING-: Specify _pwd=value in macro call ;
      %put WARNING-: or specify ETL_WEBSERVICE_pwd as system variable or global setting.;
      %let m_pwd=unknown;
   %end;

   %if ((NOT %symexist(m_url)) or (%length(&m_url)<2)) %then 
   %do;
      %put NOTE: Web service url not specified. ;
      %put NOTE-: Specify _url=value in macro call ;
      %put NOTE-: or specify ETL_WEBSERVICE_URL as system variable or global setting.;
      %let m_url=http://des18:1444/miserver/services/DataService;
   %end;

  
   %let m_encoded=&_encoded;
   %let AuthDomain=&m_AuthDomain;
   %let UserID=&m_user;
   %let PASSWORD=&m_pwd;
   
   %if &_getPWFromDomain=YES %then
   %do;
      %di_getpwfromdomain(&AuthDomain,NO,USERID,PASSWORD);
      %put NOTE: Done with di_getpwfromdomain;
      %let m_pwd=&password;
   %end;   

   %if "&m_encoded"="YES" %then 
   %do;
      %let dbpass=&m_pwd;
   %end;
   
   %else
   %do;
      filename pwfile TEMP; 

      proc pwencode in="&m_pwd" out=pwfile;
      run;


     /* save password to this macro variable */
     %local dbpass;

     /* Get macro variable from file */
      data _null_;
         infile pwfile obs=1 length=l; 
         input @;
         input @1 line $varying1024. l; 
         call symput('dbpass',substr(line,1,l)); 
      run;
      
       filename pwfile;
   %end;
    
   %put dbpass=&dbpass;

/*Save to current folder if debug on.  Otherwise in temp folder */

 %if &debug=1 %then %do;

	%let m_output_path=%di_util_first_libpath(work);
	
	%let m_action_xml  =%nrbquote(&m_output_path./action.xml);
	%let m_request_xml =%nrbquote(&m_output_path./request.xml);
	%let m_response_xml=%nrbquote(&m_output_path./response.xml);
	
   FILENAME ACTION    "&m_action_xml";
   FILENAME request   "&m_request_xml";
   FILENAME RESPONSE  "&m_response_xml";
 %end;
 %else %do;
 
   FILENAME ACTION   TEMP ;
   FILENAME request  TEMP ; 
   FILENAME RESPONSE  TEMP; 

 %end;



  /* Create a file with SOAPAction */
  DATA _NULL_;
     FILE ACTION;
     PUT ACTION @1 "SOAPAction:Ignore";
  run;



   /* If there is a table for Prod_SKs.  prod_hier_sk parameter is ignored if there is a table */
   %if %length(&prod_hier_sk_table) > 1 %then %do;
      %getProdSKs();
   %end;
   /* If there is a value for Prod_SK */
   %else %if NOT ("&prod_hier_sk"="")  %then %do;
   
      data _null_;
         call symput('m_ProdSKs', &CRLF    );
         call symput('m_ProdSKs', symget('m_ProdSKs') || '         <arg3 soapenc:arrayType="xsd:long[1]"' || &CRLF    );
         call symput('m_ProdSKs', symget('m_ProdSKs') || '               xsi:type="soapenc:Array" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || &CRLF    ); 
         call symput('m_ProdSKs', symget('m_ProdSKs') || '            <arg30 xsi:type="xsd:long">' || compress(&prod_hier_sk) || '</arg30' || '>' || &CRLF   );
         call symput('m_ProdSKs', symget('m_ProdSKs') || '         </arg3>' || &CRLF   );
      run;
                                                 
   %end;
   %else %do;


      data _null_;
         call symput('m_ProdSKs', '         <arg3 xsi:nil="true"/>' || &CRLF    );
      run;
                                                 
   
   %end;
   


 

   /* Create a Request File */
   DATA _NULL_;
      FILE request LRECL=32000;
      length temp $32000;
      PUT @1 '<?xml version="1.0" encoding="UTF-8"?>';                                          
      PUT @1 '<soapenv:Envelope';                                                               
      PUT @1 ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"';                         
      PUT @1 ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"';                                   
      PUT @1 ' xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"';                      
      PUT @1 ' xmlns:data="http://data.tkmiserver.di.solutions.sas.com">';                                   
      PUT @1 ' <soapenv:Header/>';                                                                 
      PUT @1 ' <soapenv:Body>';                                                                 
      PUT @1 "<data:runTkCmd soapenv:encodingStyle=""http://schemas.xmlsoap.org/soap/encoding/"">";    
      temp= '<arg0 xsi:type="soapenc:string" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">run=' || "&_cmd" || "</arg0>";
      PUT @10 temp;
      temp= '<arg1 xsi:type="soapenc:string" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || "&captainDir" || "</arg1>";
      PUT @10 temp;
      temp= '<arg2 xsi:type="soapenc:string" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || "&serverDir" || "</arg2>";
      PUT @10 temp;
      temp=symget('m_ProdSKs');
      put @10 temp;
         
 
      PUT request @1 "  </data:runTkCmd>";                                                                                                                                               
      PUT request @1 ' </soapenv:Body>';                                                                                                                                                
      PUT request @1 '</soapenv:Envelope>';                                                                                                                                             
   RUN;


/* Delete Response file before submitting this call.  Just so we do not capture old response */
%if %sysfunc(fileexist(response)) %then
%do;
   %put deleting response file...;
   %let sysrc=%sysfunc(fdelete(response)); 
   
   
%end;


/* Make Proc HTTP Call */
proc http 
   in=request
   out=response 
   method="post"
   headerin=ACTION 
   url="&m_url"
   webusername="&m_user"
   webpassword="&dbpass"
   webdomain="&m_AuthDomain"
   ;
run;



/* Write Request and Response to the Log */
data _null_;
   infile REQUEST ;
   input;

   put _infile_;
run;

%global processRequestReturn;

/*-------------------------------------------------------------------------------------------------------------------------*/
/* DEFECT S1270364: CRP:di_etl_export_sales_forecast does not raise errors from MerchIntelGridSvr. Modified the data step  */
/* below to capture the status code from response file. The status is sent within the <status> tag. status=0,1 indicates   */
/* successful and warning respectively and 2 is for error. The macro variable processRequestReturn will have the response  */
/* status and will be read in the di_etl_export_sales_forecast                                                             */
/*-------------------------------------------------------------------------------------------------------------------------*/

data _null_;
    infile response;
    input;
    status_tag=findc(_infile_,'>',find(_infile_,'<status'));
	if status_tag ne 0 then do;
       processRequestReturn=substr(_infile_,status_tag+1,find(_infile_,'</status')-status_tag-1);
	   call symput('processRequestReturn',compress(processRequestReturn));
	end;
	put _infile_;
run; 

  /* Return system code */
  %let _rc = &syscc;

%mend;




/*  Macro to get Product Keys */
 %macro getProdSKs();

      data _null_;

         set &prod_hier_sk_table NOBS=NOBS END=END;

      attrib key_val length=$80;
      key_val=put(prod_hier_sk, 8.);

      /* Planning Keys Header */
      if _n_=1 then do;
         call symput('m_ProdSKs', &CRLF    );
         call symput('m_ProdSKs', symget('m_ProdSKs') || '         <arg3 soapenc:arrayType="xsd:long[' || compress(put(NOBS,4.)) || ']" xsi:type="soapenc:Array"' || &CRLF    );
         call symput('m_ProdSKs', symget('m_ProdSKs') || '               xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' || &CRLF    ); 
      end;

      row=compress(put(_n_-1,4.));
      call symput('m_ProdSKs', symget('m_ProdSKs') || '            <arg3' || compress(row) || ' xsi:type="xsd:long">' || compress(put(key_val,8.)) || '</arg3' || compress(row) || '>' || &CRLF   );

      /* End of planning Keys */
      if END then do;
         call symput('m_ProdSKs', symget('m_ProdSKs') || '         </arg3>' || &CRLF   );
      end;

   run;


%mend getProdSKs;




/*   
   options mlogic mprint symbolgen;
   
libname di_data '\\midata\data\official\5.2\tests_mini';
libname di_dm '\\midata\data\official\5.2\tests_mini';

options metaport =8561
metaserver = "des18.na.sas.com"
metauser = "miuser"
metauser = "sasdemo"
metapass = "DIAdmin1"
;

%di_tkmi_request(_cmd=%str(ping;), captainDir=fcst_export,serverDir=fcst_export,_rc=, debug=1) ;
%di_tkmi_request(_cmd=%str(fcst_export;), captainDir=fcst_export,serverDir=fcst_export,_rc=, debug=1) ;
%di_tkmi_request(_cmd=%str(fa_holdout;),prod_hier_sk=,prod_hier_sk_table=, captainDir=fa_data,serverDir=fa_data,_rc=, debug=1) ;
%di_tkmi_request(_cmd=%str(fcst_export;),prod_hier_sk=,prod_hier_sk_table=, captainDir=fcst_export,serverDir=fcst_export,_rc=, debug=1) ;
%di_tkmi_request(_cmd=%str(fa_holdout;),prod_hier_sk=123,prod_hier_sk_table=, captainDir=fcst_export,serverDir=fcst_export, _rc=, debug=1) ;

* Test with a table consisting of prod_hier_sks ;
data temp;
prod_hier_sk=123; output;
prod_hier_sk=234; output;
prod_hier_sk=345; output;
prod_hier_sk=456; output;
prod_hier_sk=567; output;
prod_hier_sk=678; output;
run;

%di_tkmi_request(_cmd=%str(fa_holdout;),prod_hier_sk=123,prod_hier_sk_table=temp, captainDir=./fcst_export,serverDir=./fcst_export,_rc=resume, debug=1) ;

*/
 
